@@ -5,6 +5,7 @@ class Dataset:
     TRAIN = "train"
     TEST = "test"
     SUBSAMPLED = "subsampled"
+    PREPROCESSED = "preprocessed"
 
     def __init__(self, encoded_data=None, name: str = None, identifier: str = None, labels: dict = None):
         self.encoded_data = encoded_data
@@ -12,6 +13,11 @@ class Dataset:
         self.name = name if name is not None else self.identifier
         self.labels = labels
 
+    @classmethod
+    @abc.abstractmethod
+    def build_from_objects(cls, **kwargs):
+        pass
+
     @abc.abstractmethod
     def make_subset(self, example_indices, path, dataset_type: str):
         pass
@@ -37,5 +43,9 @@ class Dataset:
         pass
 
     @abc.abstractmethod
-    def clone(self):
+    def clone(self, keep_identifier: bool = False):
+        pass
+
+    @abc.abstractmethod
+    def get_metadata(self, field_names: list, return_df: bool = False):
         pass
