@@ -1,7 +1,6 @@
 # quality: gold
 import ast
 import logging
-import pickle
 import shutil
 import weakref
 from pathlib import Path
@@ -9,6 +8,7 @@ from typing import List
 from uuid import uuid4
 
 import numpy as np
+import yaml
 
 from immuneML.data_model.DatasetItem import DatasetItem
 from immuneML.data_model.cell.Cell import Cell
@@ -40,8 +40,8 @@ class Repertoire(DatasetItem):
     def process_custom_lists(custom_lists):
         if custom_lists:
             field_list = list(custom_lists.keys())
-            values = [custom_lists[field] for field in custom_lists.keys()]
-            dtype = [(field, np.object) for field in custom_lists.keys()]
+            values = [[NumpyHelper.get_numpy_representation(el) for el in custom_lists[field]] for field in custom_lists.keys()]
+            dtype = [(field, np.array(values[index]).dtype) for index, field in enumerate(custom_lists.keys())]
         else:
             field_list, values, dtype = [], [], []
         return field_list, values, dtype
@@ -66,12 +66,12 @@ class Repertoire(DatasetItem):
               v_subgroups: list = None, j_subgroups: list = None, v_alleles: list = None, j_alleles: list = None,
               chains: list = None, counts: list = None, region_types: list = None, frame_types: list = None,
               custom_lists: dict = None, sequence_identifiers: list = None, path: Path = None, metadata: dict = None,
-              signals: dict = None, cell_ids: list = None, filename_base: str = None):
+              signals: dict = None, cell_ids: List[str] = None, filename_base: str = None):
 
         sequence_count = Repertoire.check_count(sequence_aas, sequences, custom_lists)
 
         if sequence_identifiers is None or len(sequence_identifiers) == 0 or any(identifier is None for identifier in sequence_identifiers):
-            sequence_identifiers = list(range(sequence_count))
+            sequence_identifiers = np.arange(sequence_count).astype(str)
 
         identifier = uuid4().hex
 
@@ -92,19 +92,17 @@ class Repertoire(DatasetItem):
         for field in Repertoire.FIELDS:
             if eval(field) is not None and not all(el is None for el in eval(field)):
                 field_list.append(field)
-                values.append(eval(field))
-                dtype.append((field, np.object))
-
-        dtype = np.dtype(dtype)
+                values.append([NumpyHelper.get_numpy_representation(val) if val is not None else np.nan for val in eval(field)])
+                dtype.append((field, np.array(values[-1]).dtype))
 
         repertoire_matrix = np.array(list(map(tuple, zip(*values))), order='F', dtype=dtype)
-        np.save(data_filename, repertoire_matrix)
+        np.save(str(data_filename), repertoire_matrix, allow_pickle=False)
 
-        metadata_filename = path / f"{filename_base}_metadata.pickle"
+        metadata_filename = path / f"{filename_base}_metadata.yaml"
         metadata = {} if metadata is None else metadata
         metadata["field_list"] = field_list
-        with metadata_filename.open("wb") as file:
-            pickle.dump(metadata, file)
+        with metadata_filename.open("w") as file:
+            yaml.dump(metadata, file)
 
         repertoire = Repertoire(data_filename, metadata_filename, identifier)
         return repertoire
@@ -120,9 +118,9 @@ class Repertoire(DatasetItem):
             filename_base = filename_base if filename_base is not None else identifier
 
             data_filename = result_path / f"{filename_base}.npy"
-            np.save(data_filename, data)
+            np.save(str(data_filename), data)
 
-            metadata_filename = result_path / f"{filename_base}_metadata.pickle"
+            metadata_filename = result_path / f"{filename_base}_metadata.yaml"
             shutil.copyfile(repertoire.metadata_filename, metadata_filename)
 
             new_repertoire = Repertoire(data_filename, metadata_filename, identifier)
@@ -131,16 +129,16 @@ class Repertoire(DatasetItem):
             return None
 
     @classmethod
-    def build_from_sequence_objects(cls, sequence_objects: list, path: Path, metadata: dict):
+    def build_from_sequence_objects(cls, sequence_objects: list, path: Path, metadata: dict, filename_base: str = None):
 
         assert all(isinstance(sequence, ReceptorSequence) for sequence in sequence_objects), \
             "Repertoire: all sequences have to be instances of ReceptorSequence class."
 
         sequence_aas, sequences, v_genes, j_genes, v_subgroups, j_subgroups, v_alleles, j_alleles, chains, counts, region_types, frame_types, sequence_identifiers, cell_ids = [], [], [], [], [], [], [], [], [], [], [], [], [], []
         custom_lists = {key: [] for key in sequence_objects[0].metadata.custom_params} if sequence_objects[0].metadata else {}
-        signals = {key: [] for key in metadata if "signal" in key}
+        signals = {}
 
-        for sequence in sequence_objects:
+        for index, sequence in enumerate(sequence_objects):
             sequence_identifiers.append(sequence.identifier)
             sequence_aas.append(sequence.amino_acid_sequence)
             sequences.append(sequence.nucleotide_sequence)
@@ -158,17 +156,17 @@ class Repertoire(DatasetItem):
                 cell_ids.append(sequence.metadata.cell_id)
                 for param in sequence.metadata.custom_params.keys():
                     custom_lists[param].append(sequence.metadata.custom_params[param] if param in sequence.metadata.custom_params else None)
-            for key in signals.keys():
-                if sequence.annotation and sequence.annotation.implants and len(sequence.annotation.implants) > 0 \
-                        and "signal_" + sequence.annotation.implants[0].signal_id == key:
-                    signals[key].append(str(sequence.annotation.implants[0]))
-                else:
-                    signals[key].append(None)
+            if sequence.annotation and sequence.annotation.implants and len(sequence.annotation.implants) > 0:
+                for implant in sequence.annotation.implants:
+                    if implant.signal_id in signals:
+                        signals[implant.signal_id].append(str(implant))
+                    else:
+                        signals[implant.signal_id] = [None for _ in range(index)] + [str(implant)]
 
         return cls.build(sequence_aas=sequence_aas, sequences=sequences, v_genes=v_genes, j_genes=j_genes, v_subgroups=v_subgroups,
                          j_subgroups=j_subgroups, v_alleles=v_alleles, j_alleles=j_alleles, chains=chains, counts=counts, region_types=region_types,
                          frame_types=frame_types, custom_lists=custom_lists, sequence_identifiers=sequence_identifiers, path=path, metadata=metadata,
-                         signals=signals, cell_ids=cell_ids)
+                         signals=signals, cell_ids=cell_ids, filename_base=filename_base)
 
     def __init__(self, data_filename: Path, metadata_filename: Path, identifier: str):
         data_filename = Path(data_filename)
@@ -180,8 +178,8 @@ class Repertoire(DatasetItem):
         self.data_filename = data_filename
 
         if metadata_filename:
-            with metadata_filename.open("rb") as file:
-                self.metadata = pickle.load(file)
+            with metadata_filename.open("r") as file:
+                self.metadata = yaml.safe_load(file)
             self.fields = self.metadata["field_list"]
 
         self.metadata_filename = metadata_filename
@@ -204,7 +202,7 @@ class Repertoire(DatasetItem):
     def get_counts(self):
         counts = self.get_attribute("counts")
         if counts is not None:
-            counts = np.array([int(count) if count is not None else None for count in counts])
+            counts = np.array([int(count) if not NumpyHelper.is_nan_or_empty(count) else None for count in counts])
         return counts
 
     def get_chains(self):
@@ -215,7 +213,7 @@ class Repertoire(DatasetItem):
 
     def load_data(self):
         if self.data is None or (isinstance(self.data, weakref.ref) and self.data() is None):
-            data = np.load(self.data_filename, allow_pickle=True)
+            data = np.load(self.data_filename, allow_pickle=False)
             self.data = weakref.ref(data) if EnvironmentSettings.low_memory else data
         data = self.data() if EnvironmentSettings.low_memory else self.data
         self.element_count = data.shape[0]
@@ -256,16 +254,20 @@ class Repertoire(DatasetItem):
             self.load_data()
         return self.element_count
 
-    def _make_sequence_object(self, row):
+    def _make_sequence_object(self, row, load_implants: bool = False):
 
         fields = row.dtype.names
 
-        keys = [key for key in row.dtype.names if "signal" in key]
         implants = []
-        for key in keys:
-            value_dict = row[key]
-            if value_dict:
-                implants.append(ImplantAnnotation(**ast.literal_eval(value_dict)))
+        if load_implants:
+            keys = [key for key in row.dtype.names if key not in Repertoire.FIELDS]
+            for key in keys:
+                value_dict = row[key]
+                if value_dict:
+                    try:
+                        implants.append(ImplantAnnotation(**ast.literal_eval(value_dict)))
+                    except (SyntaxError, ValueError, TypeError) as e:
+                        pass
 
         seq = ReceptorSequence(amino_acid_sequence=row["sequence_aas"] if "sequence_aas" in fields else None,
                                nucleotide_sequence=row["sequences"] if "sequences" in fields else None,
@@ -277,7 +279,7 @@ class Repertoire(DatasetItem):
                                                          v_allele=row["v_alleles"] if "v_alleles" in fields else None,
                                                          j_allele=row["j_alleles"] if "j_alleles" in fields else None,
                                                          chain=row["chains"] if "chains" in fields else None,
-                                                         count=row["counts"] if "counts" in fields else None,
+                                                         count=row["counts"] if "counts" in fields and not NumpyHelper.is_nan_or_empty(row['counts']) else None,
                                                          region_type=row["region_types"] if "region_types" in fields else None,
                                                          frame_type=row["frame_types"] if "frame_types" in fields else "IN",
                                                          cell_id=row["cell_ids"] if "cell_ids" in fields else None,
@@ -303,18 +305,30 @@ class Repertoire(DatasetItem):
             sequences.append(self._make_sequence_object(item))
         return ReceptorBuilder.build_objects(sequences)
 
-    @property
-    def sequences(self) -> List[ReceptorSequence]:
+    def get_sequence_objects(self, load_implants: bool = False) -> List[ReceptorSequence]:
+        """
+        Lazily loads sequences from disk to reduce RAM consumption
+
+        Args:
+            load_implants: whether implants should be parsed to objects and converted to ImplantAnnotations; if True, might slow down the loading
+
+        Returns:
+            a list of ReceptorSequence objects
+        """
         seqs = []
 
         data = self.load_data()
 
         for i in range(len(self.get_sequence_identifiers())):
-            seq = self._make_sequence_object(data[i])
+            seq = self._make_sequence_object(data[i], load_implants)
             seqs.append(seq)
 
         return seqs
 
+    @property
+    def sequences(self):
+        return self.get_sequence_objects(False)
+
     @property
     def receptors(self) -> List[Receptor]:
         """
