@@ -1,4 +1,3 @@
-import pickle
 import warnings
 from multiprocessing.pool import Pool
 from pathlib import Path
@@ -7,9 +6,9 @@ from typing import List
 import numpy as np
 import pandas as pd
 
-from immuneML.IO.dataset_export.PickleExporter import PickleExporter
+from immuneML.IO.dataset_export.ImmuneMLExporter import ImmuneMLExporter
 from immuneML.IO.dataset_import.DatasetImportParams import DatasetImportParams
-from immuneML.IO.dataset_import.PickleImport import PickleImport
+from immuneML.IO.dataset_import.ImmuneMLImport import ImmuneMLImport
 from immuneML.data_model.dataset import Dataset
 from immuneML.data_model.dataset.ReceptorDataset import ReceptorDataset
 from immuneML.data_model.dataset.RepertoireDataset import RepertoireDataset
@@ -61,7 +60,7 @@ class ImportHelper:
 
         if dataset_file.is_file():
             params["path"] = dataset_file
-            dataset = PickleImport.import_dataset(params, dataset_name)
+            dataset = ImmuneMLImport.import_dataset(params, dataset_name)
 
         return dataset
 
@@ -77,8 +76,14 @@ class ImportHelper:
 
         Returns:
             RepertoireDataset object that was created
+
         """
-        metadata = pd.read_csv(params.metadata_file, ",")
+
+        try:
+            metadata = pd.read_csv(params.metadata_file, ",")
+        except Exception as e:
+            raise Exception(f"{e}\nAn error occurred while reading in the metadata file {params.metadata_file}. Please see the error log above for "
+                            f"more details on this error and the documentation for the expected format of the metadata.")
 
         ParameterValidator.assert_keys_present(metadata.columns.tolist(), ["filename"], ImportHelper.__name__,
                                                f'{dataset_name}: params: metadata_file')
@@ -95,20 +100,44 @@ class ImportHelper:
         dataset = RepertoireDataset(labels={key: list(set(metadata[key].values.tolist())) for key in potential_labels},
                                     repertoires=repertoires, metadata_file=new_metadata_file, name=dataset_name)
 
-        PickleExporter.export(dataset, params.result_path)
+        ImmuneMLExporter.export(dataset, params.result_path)
 
         return dataset
 
     @staticmethod
     def update_gene_info(df: pd.DataFrame):
+        """
+        Updates gene info in 2 steps:
+        - First, columns are added if they were not present. This is done by going from the highest level of information (alleles)
+          towards the lowest level of information (subgroups) by stripping away suffixes. If gene and subgroup columns were already
+          present, suffixes are still stripped away just in case.
+        - Next, if there are None values present, the highest possible level of information is copied in from the lower level information fields.
+          This is done by moving from subgroups towards alleles. So if for one particular receptor only the subgroup was present, the subgroup
+          will be copied into the genes and alleles column.
+        """
         for gene in ['v', 'j']:
-            if f"{gene}_genes" in df.columns:
-                df.loc[:, f"{gene}_alleles"] = df[f"{gene}_genes"]
-                df.loc[:, f"{gene}_genes"] = ImportHelper.strip_alleles(df, f"{gene}_genes")
-                if f'{gene}_subgroups' not in df.columns:
-                    df.loc[:, f'{gene}_subgroups'] = [item.split("-")[0] if item is not None else None for item in df[f"{gene}_genes"]]
-            elif f"{gene}_alleles" in df.columns and f"{gene}_genes" not in df.columns:
+            # step 1: create all columns
+            if f"{gene}_alleles" in df.columns and not f"{gene}_genes" in df.columns:
                 df.loc[:, f"{gene}_genes"] = ImportHelper.strip_alleles(df, f"{gene}_alleles")
+
+            if f"{gene}_genes" in df.columns:
+                df.loc[:, f"{gene}_genes"] = ImportHelper.strip_alleles(df, f"{gene}_genes")
+                if not f"{gene}_subgroups" in df.columns:
+                    df.loc[:, f"{gene}_subgroups"] = ImportHelper.strip_genes(df, f"{gene}_genes")
+            elif f"{gene}_subgroups" in df.columns:
+                df.loc[:, f"{gene}_subgroups"] = ImportHelper.strip_genes(df, f"{gene}_subgroups")
+
+            for type in ["alleles", "genes", "subgroups"]:
+                if f"{gene}_{type}" not in df.columns:
+                    df[f"{gene}_{type}"] = None
+
+            # step 2: fill in missing info
+            missing_gene = df[f"{gene}_genes"].isnull()
+            missing_allele = df[f"{gene}_alleles"].isnull()
+
+            df.loc[missing_gene, f"{gene}_genes"] = df.loc[missing_gene, f"{gene}_subgroups"]
+            df.loc[missing_allele, f"{gene}_alleles"] = df.loc[missing_allele, f"{gene}_genes"]
+
         return df
 
     @staticmethod
@@ -124,7 +153,6 @@ class ImportHelper:
 
     @staticmethod
     def load_repertoire_as_object(import_class, metadata_row, params: DatasetImportParams):
-        alternative_load_func = getattr(import_class, "alternative_load_func", None)
         try:
             alternative_load_func = getattr(import_class, "alternative_load_func", None)
 
@@ -151,26 +179,59 @@ class ImportHelper:
             if alternative_load_func:
                 df = alternative_load_func(filepath, params)
             else:
-                df = pd.read_csv(filepath, sep=params.separator, iterator=False, usecols=params.columns_to_load, dtype=str)
+                df = ImportHelper.safe_load_dataframe(filepath, params)
         except Exception as ex:
             raise Exception(f"{ex}\n\nImportHelper: an error occurred during dataset import while parsing the input file: {filepath}.\n"
                             f"Please make sure this is a correct immune receptor data file (not metadata).\n"
-                            f"The parameters used for import are {params}.\nFor technical description of the error, see the log above."
-                            f" For details on how to specify the dataset import, see the documentation.")
+                            f"The parameters used for import are {params}.\nFor technical description of the error, see the log above. "
+                            f"For details on how to specify the dataset import, see the documentation.")
 
-        if hasattr(params, "column_mapping") and params.column_mapping is not None:
-            df.rename(columns=params.column_mapping, inplace=True)
-
-        if hasattr(params, "metadata_column_mapping") and params.metadata_column_mapping is not None:
-            df.rename(columns=params.metadata_column_mapping, inplace=True)
-
-        df = ImportHelper.standardize_none_values(df)
+        ImportHelper.rename_dataframe_columns(df, params)
+        ImportHelper.standardize_none_values(df)
 
         return df
 
     @staticmethod
-    def standardize_none_values(dataframe: pd.DataFrame) -> pd.DataFrame:
-        return dataframe.replace({key: Constants.UNKNOWN for key in ["unresolved", "no data", "na", "unknown", "null", "nan", np.nan, ""]})
+    def safe_load_dataframe(filepath, params: DatasetImportParams):
+        if hasattr(params, "columns_to_load") and params.columns_to_load is not None:
+            usecols = set(params.columns_to_load) if hasattr(params, "columns_to_load") and params.columns_to_load is not None else set()
+            usecols = usecols.union(
+                set(params.column_mapping.keys()) if hasattr(params, "column_mapping") and params.column_mapping is not None else set())
+            usecols = usecols.union(set(params.column_mapping_synonyms.keys())
+                                    if hasattr(params, "column_mapping_synonyms") and params.column_mapping_synonyms is not None else set())
+            usecols = usecols.union(set(params.metadata_column_mapping.keys())
+                                    if hasattr(params, "metadata_column_mapping") and params.metadata_column_mapping is not None else set())
+        else:
+            usecols = None
+
+        try:
+            df = pd.read_csv(filepath, sep=params.separator, iterator=False, usecols=usecols, dtype=str)
+        except ValueError:
+            try:
+                df = pd.read_csv(filepath, sep=params.separator, iterator=False, usecols=params.columns_to_load, dtype=str)
+            except ValueError:
+                df = pd.read_csv(filepath, sep=params.separator, iterator=False, dtype=str)
+                warnings.warn(f"ImportHelper: failed to import columns {params.columns_to_load} for "
+                              f"the input file {filepath}, imported the following instead: {list(df.columns)}")
+
+        return df
+
+    @staticmethod
+    def rename_dataframe_columns(df, params: DatasetImportParams):
+        if hasattr(params, "column_mapping") and params.column_mapping is not None:
+            df.rename(columns=params.column_mapping, inplace=True)
+
+        if hasattr(params, "column_mapping_synonyms") and params.column_mapping_synonyms is not None:
+            for synonym, colname in params.column_mapping_synonyms.items():
+                if colname not in df.columns:
+                    df.rename(columns={synonym: colname}, inplace=True)
+
+        if hasattr(params, "metadata_column_mapping") and params.metadata_column_mapping is not None:
+            df.rename(columns=params.metadata_column_mapping, inplace=True)
+
+    @staticmethod
+    def standardize_none_values(dataframe: pd.DataFrame):
+        dataframe.replace({key: Constants.UNKNOWN for key in ["unresolved", "no data", "na", "unknown", "null", "nan", np.nan, ""]}, inplace=True)
 
     @staticmethod
     def drop_empty_sequences(dataframe: pd.DataFrame, import_empty_aa_sequences: bool, import_empty_nt_sequences: bool) -> pd.DataFrame:
@@ -185,14 +246,16 @@ class ImportHelper:
             sequence_name = sequence_type.name.lower().replace("_", " ")
 
             if sequence_colname in dataframe.columns:
+                dataframe[sequence_colname].replace({"": Constants.UNKNOWN}, inplace=True)
+
                 n_empty = sum(dataframe[sequence_colname].isnull())
                 if n_empty > 0:
-                    idx = dataframe.loc[dataframe[sequence_colname].isnull()].index
                     dataframe.drop(dataframe.loc[dataframe[sequence_colname].isnull()].index, inplace=True)
                     warnings.warn(
-                        f"{ImportHelper.__name__}: {n_empty} sequences were removed from the dataset because they contained an empty {sequence_name} sequence after preprocessing. ")
+                        f"{ImportHelper.__name__}: {n_empty} sequences were removed from the dataset because they contained an empty {sequence_name} "
+                        f"sequence after preprocessing. ")
             else:
-                warnings.warn(f"{ImportHelper.__name__}: column {sequence_colname} was not set, skipping filtering...")
+                raise ValueError(f"{ImportHelper.__name__}: column {sequence_colname} was not set, but is required for filtering.")
 
         return dataframe
 
@@ -235,24 +298,26 @@ class ImportHelper:
         return frame_type_list
 
     @staticmethod
-    def load_chains(df: pd.DataFrame, column_name="chains") -> list:
-        return [Chain.get_chain(chain_str).value if chain_str is not None else None for chain_str in df[column_name]]
+    def load_chains(df: pd.DataFrame):
+        if "chains" in df.columns:
+            df.loc[:, "chains"] = ImportHelper.load_chains_from_chains(df)
+        else:
+            df.loc[:, "chains"] = ImportHelper.load_chains_from_genes(df)
+
+    @staticmethod
+    def load_chains_from_chains(df: pd.DataFrame) -> list:
+        return [Chain.get_chain(chain_str).value if chain_str is not None else None for chain_str in df["chains"]]
 
     @staticmethod
     def load_chains_from_genes(df: pd.DataFrame) -> list:
-        result = None
-
-        columns_with_gene_info = ["v_genes", "j_genes", "v_alleles", "j_alleles"]
-        present_columns = [col for col in columns_with_gene_info if col in df.columns]
-
-        if len(present_columns) > 0:
-            result = ImportHelper.load_chains_from_column(df, present_columns[0])
-
-        return result
+        return df.apply(ImportHelper.get_chain_for_row, axis=1)
 
     @staticmethod
-    def load_chains_from_column(df: pd.DataFrame, column_name) -> list:
-        return [Chain.get_chain(chain_str).value if chain_str is not None else None for chain_str in df[column_name].str[0:3]]
+    def get_chain_for_row(row):
+        for col in ["v_subgroup", "j_subgroup", "v_genes", "j_genes", "v_alleles", "j_alleles"]:
+            if col in row and row[col] is not None:
+                return Chain.get_chain(str(row[col])[0:3]).value
+        return None
 
     @staticmethod
     def junction_to_cdr3(df: pd.DataFrame, region_type: RegionType):
@@ -270,11 +335,19 @@ class ImportHelper:
 
     @staticmethod
     def strip_alleles(df: pd.DataFrame, column_name):
+        return ImportHelper.strip_suffix(df, column_name, Constants.ALLELE_DELIMITER)
+
+    @staticmethod
+    def strip_genes(df: pd.DataFrame, column_name):
+        return ImportHelper.strip_suffix(df, column_name, Constants.GENE_DELIMITER)
+
+    @staticmethod
+    def strip_suffix(df: pd.DataFrame, column_name, delimiter):
         """
-        Removes alleles (everything after the '*' character) from a column in the DataFrame
+        Safely removes everything after a delimiter from a column in the DataFrame
         """
         if column_name in df.columns:
-            return df[column_name].apply(lambda gene_col: None if gene_col is None else gene_col.split(Constants.ALLELE_DELIMITER)[0])
+            return df[column_name].apply(lambda gene_col: None if gene_col is None else gene_col.rsplit(delimiter)[0])
 
     @staticmethod
     def get_sequence_filenames(path: Path, dataset_name: str):
@@ -305,23 +378,26 @@ class ImportHelper:
         dataset_filenames = []
         dataset_params = {}
         items = None
+        class_name = None
 
         for index, filename in enumerate(filenames):
             new_items = ImportHelper.import_items(import_class, filename, params)
             items = np.append(items, new_items) if items is not None else new_items
             dataset_params = ImportHelper.extract_sequence_dataset_params(items, params)
+            class_name = type(new_items[0]).__name__ if len(new_items) > 0 else None
 
             while len(items) > params.sequence_file_size or (index == len(filenames) - 1 and len(items) > 0):
-                dataset_filenames.append(params.result_path / "batch_{}.pickle".format(file_index))
+                dataset_filenames.append(params.result_path / "batch_{}.npy".format(file_index))
                 ImportHelper.store_sequence_items(dataset_filenames, items, params.sequence_file_size)
                 items = items[params.sequence_file_size:]
                 file_index += 1
 
-        init_kwargs = {"filenames": dataset_filenames, "file_size": params.sequence_file_size, "name": dataset_name, "labels": dataset_params}
+        init_kwargs = {"filenames": dataset_filenames, "file_size": params.sequence_file_size, "name": dataset_name, "labels": dataset_params,
+                       "element_class_name": class_name}
 
         dataset = ReceptorDataset(**init_kwargs) if params.paired else SequenceDataset(**init_kwargs)
 
-        PickleExporter.export(dataset, params.result_path)
+        ImmuneMLExporter.export(dataset, params.result_path)
 
         return dataset
 
@@ -360,8 +436,9 @@ class ImportHelper:
 
     @staticmethod
     def store_sequence_items(dataset_filenames: list, items: list, sequence_file_size: int):
-        with dataset_filenames[-1].open("wb") as file:
-            pickle.dump(items[:sequence_file_size], file)
+        sequence_matrix = np.core.records.fromrecords([item.get_record() for item in items[:sequence_file_size]],
+                                                      names=type(items[0]).get_record_names())
+        np.save(str(dataset_filenames[-1]), sequence_matrix, allow_pickle=False)
 
     @staticmethod
     def import_sequence(row, metadata_columns=None) -> ReceptorSequence:
@@ -388,60 +465,66 @@ class ImportHelper:
     @staticmethod
     def import_receptors(df, params) -> List[Receptor]:
         identifiers = df["receptor_identifiers"].unique()
+
+        chain_pair = params.receptor_chains
+        if chain_pair is None:
+            chains = [Chain.get_chain(chain) for chain in df["chains"].unique()]
+            chain_pair = ChainPair.get_chain_pair(chains)
+
+        metadata_columns = list(params.metadata_column_mapping.values()) if params.metadata_column_mapping else None
+
         all_receptors = []
 
         for identifier in identifiers:
-            receptors = ImportHelper.import_receptors_by_id(df, identifier, params)
+            receptors = ImportHelper.import_receptors_by_id(df, identifier, chain_pair, metadata_columns)
             all_receptors.extend(receptors)
 
         return all_receptors
 
     @staticmethod
-    def import_receptors_by_id(df, identifier, params) -> List[Receptor]:
-        first_row = df.loc[(df["receptor_identifiers"] == identifier) & (df["chains"] == params.receptor_chains.value[0])]
-        second_row = df.loc[(df["receptor_identifiers"] == identifier) & (df["chains"] == params.receptor_chains.value[1])]
+    def import_receptors_by_id(df, identifier, chain_pair, metadata_columns) -> List[Receptor]:
+        first_row = df.loc[(df["receptor_identifiers"] == identifier) & (df["chains"] == chain_pair.value[0])]
+        second_row = df.loc[(df["receptor_identifiers"] == identifier) & (df["chains"] == chain_pair.value[1])]
 
         for i, row in enumerate([first_row, second_row]):
             if row.shape[0] > 1:
                 warnings.warn(
-                    f"Multiple {params.receptor_chains.value[i]} chains found for receptor with identifier {identifier}, only the first entry will be loaded")
+                    f"Multiple {chain_pair.value[i]} chains found for receptor with identifier {identifier}, only the first entry will be loaded")
             elif row.shape[0] == 0:
                 warnings.warn(
-                    f"Missing {params.receptor_chains.value[i]} chain for receptor with identifier {identifier}, this receptor will be omitted.")
+                    f"Missing {chain_pair.value[i]} chain for receptor with identifier {identifier}, this receptor will be omitted.")
                 return []
 
-        # todo add options like IRIS import: option to import all dual chains or just the first pair / all V genes when uncertain annotation, etc
         # todo add possibility to import multiple chain combo's? (BCR heavy-light & heavy-kappa, as seen in 10xGenomics?)
 
-        return [ImportHelper.build_receptor_from_rows(first_row.iloc[0], second_row.iloc[0], identifier, params)]
+        return [ImportHelper.build_receptor_from_rows(first_row.iloc[0], second_row.iloc[0], identifier, chain_pair, metadata_columns)]
 
     @staticmethod
-    def build_receptor_from_rows(first_row, second_row, identifier, params):
-        metadata_columns = params.metadata_column_mapping.values() if params.metadata_column_mapping else None
+    def build_receptor_from_rows(first_row, second_row, identifier, chain_pair, metadata_columns):
         first_sequence = ImportHelper.import_sequence(first_row, metadata_columns=metadata_columns)
         second_sequence = ImportHelper.import_sequence(second_row, metadata_columns=metadata_columns)
 
-        if params.receptor_chains == ChainPair.TRA_TRB:
+        if chain_pair == ChainPair.TRA_TRB:
             receptor = TCABReceptor(alpha=first_sequence,
                                     beta=second_sequence,
                                     identifier=identifier,
                                     metadata={**second_sequence.metadata.custom_params})
-        elif params.receptor_chains == ChainPair.TRG_TRD:
+        elif chain_pair == ChainPair.TRG_TRD:
             receptor = TCGDReceptor(gamma=first_sequence,
                                     delta=second_sequence,
                                     identifier=identifier,
                                     metadata={**second_sequence.metadata.custom_params})
-        elif params.receptor_chains == ChainPair.IGH_IGL:
+        elif chain_pair == ChainPair.IGH_IGL:
             receptor = BCReceptor(heavy=first_sequence,
                                   light=second_sequence,
                                   identifier=identifier,
                                   metadata={**first_sequence.metadata.custom_params})
-        elif params.receptor_chains == ChainPair.IGH_IGK:
+        elif chain_pair == ChainPair.IGH_IGK:
             receptor = BCKReceptor(heavy=first_sequence,
                                    kappa=second_sequence,
                                    identifier=identifier,
                                    metadata={**first_sequence.metadata.custom_params})
         else:
-            raise NotImplementedError(f"ImportHelper: {params.receptor_chains} chain pair is not supported.")
+            raise NotImplementedError(f"ImportHelper: {chain_pair} chain pair is not supported.")
 
         return receptor
