@@ -6,13 +6,13 @@ import pandas as pd
 
 from immuneML.IO.ml_method.MLExporter import MLExporter
 from immuneML.environment.LabelConfiguration import LabelConfiguration
-from immuneML.environment.Metric import Metric
 from immuneML.hyperparameter_optimization.config.SplitConfig import SplitConfig
 from immuneML.hyperparameter_optimization.config.SplitType import SplitType
 from immuneML.hyperparameter_optimization.core.HPAssessment import HPAssessment
 from immuneML.hyperparameter_optimization.core.HPUtil import HPUtil
 from immuneML.hyperparameter_optimization.states.TrainMLModelState import TrainMLModelState
 from immuneML.hyperparameter_optimization.strategy.HPOptimizationStrategy import HPOptimizationStrategy
+from immuneML.ml_metrics.Metric import Metric
 from immuneML.reports.train_ml_model_reports.TrainMLModelReport import TrainMLModelReport
 from immuneML.util.ReflectionHandler import ReflectionHandler
 from immuneML.workflows.instructions.Instruction import Instruction
@@ -61,14 +61,11 @@ class TrainMLModelInstruction(Instruction):
         number_of_processes (int): how many processes should be created at once to speed up the analysis. For personal machines, 4 or 8 is usually a good choice.
 
         reports (list): a list of report names to be executed after the nested CV has finished to show the overall performance or some statistic;
-        the reports to be specified here have to be :py:obj:`immuneML.reports.train_ml_model_reports.TrainMLModelReport.TrainMLModelReport` reports.
+        the reports to be specified here have to be :py:obj:`~immuneML.reports.train_ml_model_reports.TrainMLModelReport.TrainMLModelReport` reports.
 
         refit_optimal_model (bool): if the final combination of preprocessing-encoding-ML model should be refitted on the full dataset thus providing
         the final model to be exported from instruction; alternatively, train combination from one of the assessment folds will be used
 
-        store_encoded_data (bool): if the encoded datasets should be stored, can be True or False; setting this argument to True might increase the
-        disk usage significantly
-
     YAML specification:
 
     .. indent with spaces
@@ -118,16 +115,15 @@ class TrainMLModelInstruction(Instruction):
             number_of_processes: 4 # number of parallel processes to create (could speed up the computation)
             optimization_metric: balanced_accuracy # the metric to use for choosing the optimal model and during training
             refit_optimal_model: False # use trained model, do not refit on the full dataset
-            store_encoded_data: True # store encoded datasets in pickle format
 
     """
 
     def __init__(self, dataset, hp_strategy: HPOptimizationStrategy, hp_settings: list, assessment: SplitConfig, selection: SplitConfig,
                  metrics: set, optimization_metric: Metric, label_configuration: LabelConfiguration, path: Path = None, context: dict = None,
-                 number_of_processes: int = 1, reports: dict = None, name: str = None, refit_optimal_model: bool = False, store_encoded_data: bool = None):
+                 number_of_processes: int = 1, reports: dict = None, name: str = None, refit_optimal_model: bool = False):
         self.state = TrainMLModelState(dataset, hp_strategy, hp_settings, assessment, selection, metrics,
                                        optimization_metric, label_configuration, path, context, number_of_processes,
-                                       reports if reports is not None else {}, name, refit_optimal_model, store_encoded_data)
+                                       reports if reports is not None else {}, name, refit_optimal_model)
 
     def run(self, result_path: Path):
         self.state.path = result_path
@@ -153,8 +149,7 @@ class TrainMLModelInstruction(Instruction):
             print(f"{datetime.datetime.now()}: TrainMLModel: retraining optimal model for label {label} {index_repr}.\n", flush=True)
             self.state.optimal_hp_items[label] = MLProcess(self.state.dataset, None, label, self.state.metrics, self.state.optimization_metric,
                                                            self.state.path / f"optimal_{label}", number_of_processes=self.state.number_of_processes,
-                                                           label_config=self.state.label_configuration, hp_setting=optimal_hp_setting,
-                                                           store_encoded_data=self.state.store_encoded_data).run(0)
+                                                           label_config=self.state.label_configuration, hp_setting=optimal_hp_setting).run(0)
             print(f"{datetime.datetime.now()}: TrainMLModel: finished retraining optimal model for label {label} {index_repr}.\n", flush=True)
 
         else:
@@ -234,7 +229,7 @@ class TrainMLModelInstruction(Instruction):
             "a metric to use for optimization": f"a metric to use for optimization (one of {valid_values})",
             "Valid values are objects of any class inheriting :py:obj:`~immuneML.hyperparameter_optimization.strategy."
             "HPOptimizationStrategy.HPOptimizationStrategy`.": f"Valid values are: {valid_strategies}.",
-            "the reports to be specified here have to be :py:obj:`immuneML.reports.train_ml_model_reports.TrainMLModelReport.TrainMLModelReport` reports.": f"the reports that can be provided here are :ref:`{TrainMLModelReport.get_title()}`."
+            "the reports to be specified here have to be :py:obj:`~immuneML.reports.train_ml_model_reports.TrainMLModelReport.TrainMLModelReport` reports.": f"the reports that can be provided here are :ref:`{TrainMLModelReport.get_title()}`."
         }
         doc = update_docs_per_mapping(doc, mapping)
         return doc
