@@ -1,6 +1,7 @@
 from dataclasses import dataclass
 
 from immuneML.data_model.dataset.Dataset import Dataset
+from immuneML.environment.LabelConfiguration import LabelConfiguration
 from immuneML.hyperparameter_optimization.config.SplitConfig import SplitConfig
 from immuneML.hyperparameter_optimization.config.SplitType import SplitType
 from immuneML.workflows.steps.StepParams import StepParams
@@ -15,3 +16,4 @@ class DataSplitterParams(StepParams):
     training_percentage: float = -1
     paths: list = None
     split_config: SplitConfig = None
+    label_config: LabelConfiguration = None
